==> youamp_0.4.1-2.tar.gz: youamp-0.4.1/youamp/ui/searchview.py <==
import gtk
import pango
import hildon

from youamp.ui.browser import Browser
from youamp.ui import NOTHING_SELECTED, ARTIST_SELECTED, ALBUM_SELECTED

class BrowseButton(gtk.Button):
    def __init__(self):
        gtk.Button.__init__(self)
        
        self.set_size_request(200, -1)
        
        self.label = gtk.Label()
        self.label.set_ellipsize(pango.ELLIPSIZE_END)
        
        self.add(self.label)
        self.unset_flags(gtk.CAN_FOCUS)
    
    def set_text(self, txt):
        self.label.set_text(txt)

class SearchView(gtk.VBox):
    def __init__(self, config, player, library):
        gtk.VBox.__init__(self)
        self.set_spacing(5)

        self._config = config
        self._library = library
        self._player = player
        self._is_browser = self._config["is-browser"]
        
        # Navi Controls
        navi = gtk.HBox()
        navi.set_spacing(5)

        # Search Label
        self._search_label = gtk.Label(_("Search"))
        self._search_label.set_size_request(100, -1)
        navi.pack_start(self._search_label, expand=False)
        
        # Browse Button
        self._browse_button = gtk.Button(_("Browse"))
        self._browse_button.connect("clicked", self._show_artists)
        self._browse_button.set_size_request(100, -1)
        self._browse_button.unset_flags(gtk.CAN_FOCUS)
        navi.pack_start(self._browse_button, expand=False)
        
        # Navi Artist Button
        self._artist = BrowseButton()
        self._artist.connect("clicked", self._show_albums)
        navi.pack_start(self._artist, expand=False)
        
        # Navi Album Button
        self._album = BrowseButton()
        self._album.connect("clicked", self._select_album)
        navi.pack_start(self._album, expand=False)
        
        # Navi Search Entry
        self._search_entry = gtk.Entry()
        self._search_entry.connect("activate", self._on_search_click)
        self._search_entry.set_text(self._config["search-str"])
        navi.pack_start(self._search_entry, expand=True)
        
        # Shuffle Button
        self.shuffle = gtk.ToggleButton(_("shuffle"))
        self.shuffle.unset_flags(gtk.CAN_FOCUS)
        self.shuffle.set_active(self._config["shuffle"])
        self.shuffle.connect("toggled", self._on_shuffle_toggled)
        
        navi.pack_end(self.shuffle, expand=False)
        self.pack_start(navi, expand=False)
        
        # Scrolled View
        self._scroll = gtk.ScrolledWindow()
        self._scroll.set_policy(gtk.POLICY_NEVER, gtk.POLICY_ALWAYS)
        self._scroll.set_shadow_type(gtk.SHADOW_IN)
        hildon.hildon_helper_set_thumb_scrollbar(self._scroll, True)
        
        self._playlist = PlaylistView(player.playlist, self._config, self._player)        
        self._browser = Browser(self._config, self._library)
        
        self._scroll.add(self._playlist)

        self.pack_start(self._scroll)

        self.show_all() 
 
        # Restore View
        if self._config["is-browser"]:
            a = self._config["search-artist"]
            txt = a if a != "" else _("All Artists")
            self._artist.set_text(txt)   
            
            a = self._config["search-album"]
            txt = a if a !=  "" else _("All Albums")                
            self._album.set_text(txt)

            self._browser._state = ARTIST_SELECTED | ALBUM_SELECTED
            self._search_label.hide()
            self._search_entry.hide()
        else:
            self._browse_button.hide()
            self._artist.hide()
            self._album.hide()

   
        # browser callbacks
        self._config.notify_add("search-artist", self._on_search_artist)
        self._config.notify_add("search-album", self._on_search_album)

    def _on_shuffle_toggled(self, caller):
        self._config["shuffle"] = not self._config["shuffle"]
        self._player.playlist.shuffle(self._config["shuffle"])
        
        self._config["pos"] = self._player.playlist.get_new_pos(self._config["pos"])
        
    def _browse_complete(self):        
        self._search_entry.set_text("")
        self._config["is-browser"] = True
        
        playlist = self._library.get_tracks(self._config)
        self._set_playlist(playlist)
        
        self._show_playlist()
 
    def _on_search_click(self, caller):
        self._config["search-str"] = caller.get_text()
        self._config["shuffle"] = self.shuffle.get_active()
        self._config["is-browser"] = False
        
        playlist = self._library.get_tracks(self._config)
        
        self._set_playlist(playlist)
        
        self._config.notify("search-str")

    def _set_playlist(self, data):
        self._config["pos"] = 0

        self._player.playlist.update(data)
        
        if self._config["shuffle"]:
            self._player.playlist.shuffle(True)
    
    def browse_mode(self):
        self._search_label.hide()
        self._search_entry.hide()
        self._browse_button.show()
        
        if self._config["is-browser"]:
            self._artist.show()
            self._album.show()
        else:
            self._show_artists()
    
    def search_mode(self):
        self._browse_button.hide()
        self._artist.hide()
        self._album.hide()
        self._search_label.show()
        self._search_entry.show()
        
        if self._browser.get_parent() is not None:
            self._show_playlist()
    
    def _show_playlist(self):
        if self._playlist.get_parent() is None:
            self._scroll.remove(self._browser)
            self._scroll.add(self._playlist)

    def _show_browser(self):
        if self._browser.get_parent() is None:
            self._scroll.remove(self._playlist)
            self._scroll.add(self._browser)

    def _on_search_artist(self, client, cxn_id, entry, data):
        val = entry.get_value().get_string()

        val = val if val != "" else _("All Artists")
        
        self._artist.set_text(val)
        self._artist.show()

        self._album.hide()
        print "search-artist changed"

    def _on_search_album(self, client, cxn_id, entry, data):
        val = entry.get_value().get_string()
  
        val = val if val != "" else _("All Albums")
        
        self._album.set_text(val)
        self._album.show()
        
        self._browse_complete()
        print "search-album changed"

    def _select_album(self, caller):        
        if not (self._browser._state & ALBUM_SELECTED):
            self._browser._state |= ALBUM_SELECTED
            self._config.notify("search-album")
        
    def _show_albums(self, caller):
        if not (self._browser._state & ALBUM_SELECTED):
            self._config.notify("search-artist")

        self._show_browser()
        self._browser.show_albums()
        
        self._browser._state = ARTIST_SELECTED

    def _show_artists(self, caller=None):
        self._show_browser()
        self._browser.show_artists()
        self._browser._state = NOTHING_SELECTED

class PlaylistView(gtk.TreeView):
    def __init__(self, model, config, player):        
        gtk.TreeView.__init__(self, model)
        
        self._config = config
        self._player = player
        
        self.set_rules_hint(True)
        self.set_headers_visible(True)
        self.set_fixed_height_mode(True)
        keys = ("title", "artist", "album")
        cols = (_("Title"), _("Artist"), _("Album"))
        
        cell = gtk.CellRendererText()
        cell.set_property("ellipsize", pango.ELLIPSIZE_END)
            
        for i in xrange(len(keys)):
            col = gtk.TreeViewColumn(cols[i], cell)
            col.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
            col.set_fixed_width(200)
            col.set_cell_data_func(cell, self._data_func, keys[i])
            self.append_column(col)
        
        # Signals
        self._config.notify_add("pos", self._on_pos_changed)
        self.connect("row-activated", self._on_row_activated)

    def _on_pos_changed(self, *args):
        self.set_cursor(self._config["pos"])
 
    def _on_row_activated(self, caller, path, column):
        self._player.goto_pos(path[0])

    def _data_func(self, col, cell, model, itr, key):
        cell.set_property("text", model[itr][key])

==> youamp_0.4.1-3.tar.gz: youamp-0.4.1/youamp/ui/searchview.py <==
import gtk
import gtk.gdk
import pango
import hildon

from youamp.ui.browser import Browser
from youamp.ui import NOTHING_SELECTED, ARTIST_SELECTED, ALBUM_SELECTED
from youamp import IS_MAEMO

class BrowseButton(gtk.Button):
    def __init__(self):
        gtk.Button.__init__(self)
        
        self.set_size_request(200, -1)
        
        self.label = gtk.Label()
        self.label.set_ellipsize(pango.ELLIPSIZE_END)
        
        self.add(self.label)
        self.unset_flags(gtk.CAN_FOCUS)
    
    def set_text(self, txt):
        self.label.set_text(txt)

class SearchView(gtk.VBox):
    def __init__(self, config, player, library):
        gtk.VBox.__init__(self)
        self.set_spacing(5)

        self._config = config
        self._library = library
        self._player = player
        self._is_browser = self._config["is-browser"]
        
        # Navi Controls
        navi = gtk.HBox()
        navi.set_spacing(5)

        # Search Label
        self._search_label = gtk.Label(_("Search"))
        self._search_label.set_size_request(100, -1)
        navi.pack_start(self._search_label, expand=False)
        
        # Browse Button
        self._browse_button = gtk.Button(_("Browse"))
        self._browse_button.connect("clicked", self._show_artists)
        self._browse_button.set_size_request(100, -1)
        self._browse_button.unset_flags(gtk.CAN_FOCUS)
        navi.pack_start(self._browse_button, expand=False)
        
        # Navi Artist Button
        self._artist = BrowseButton()
        self._artist.connect("clicked", self._show_albums)
        navi.pack_start(self._artist, expand=False)
        
        # Navi Album Button
        self._album = BrowseButton()
        self._album.connect("clicked", self._select_album)
        navi.pack_start(self._album, expand=False)
        
        # Navi Search Entry
        self._search_entry = gtk.Entry()
        self._search_entry.connect("activate", self._on_search_click)
        self._search_entry.set_text(self._config["search-str"])
        navi.pack_start(self._search_entry, expand=True)
        
        # Shuffle Button
        self.shuffle = gtk.ToggleButton(_("shuffle"))
        self.shuffle.unset_flags(gtk.CAN_FOCUS)
        self.shuffle.set_active(self._config["shuffle"])
        self.shuffle.connect("toggled", self._on_shuffle_toggled)
        
        navi.pack_end(self.shuffle, expand=False)
        self.pack_start(navi, expand=False)
        
        # Scrolled View
        self._scroll = gtk.ScrolledWindow()
        self._scroll.set_policy(gtk.POLICY_NEVER, gtk.POLICY_ALWAYS)
        self._scroll.set_shadow_type(gtk.SHADOW_IN)
        hildon.hildon_helper_set_thumb_scrollbar(self._scroll, True)
        
        self._playlist = PlaylistView(player.playlist, self._config, self._player)        
        self._browser = Browser(self._config, self._library)
        
        self._scroll.add(self._playlist)

        self.pack_start(self._scroll)

        self.show_all() 
 
        # Restore View
        if self._config["is-browser"]:
            a = self._config["search-artist"]
            txt = a if a != "" else _("All Artists")
            self._artist.set_text(txt)   
            
            a = self._config["search-album"]
            txt = a if a !=  "" else _("All Albums")                
            self._album.set_text(txt)

            self._browser._state = ARTIST_SELECTED | ALBUM_SELECTED
            self._search_label.hide()
            self._search_entry.hide()
        else:
            self._browse_button.hide()
            self._artist.hide()
            self._album.hide()

   
        # browser callbacks
        self._config.notify_add("search-artist", self._on_search_artist)
        self._config.notify_add("search-album", self._on_search_album)

    def _on_shuffle_toggled(self, caller):
        self._config["shuffle"] = not self._config["shuffle"]
        self._player.playlist.shuffle(self._config["shuffle"])
        
        self._config["pos"] = self._player.playlist.get_new_pos(self._config["pos"])
        
    def _browse_complete(self):        
        self._search_entry.set_text("")
        self._config["is-browser"] = True
        
        playlist = self._library.get_tracks(self._config)
        self._set_playlist(playlist)
        
        self._show_playlist()
 
    def _on_search_click(self, caller):
        self._config["search-str"] = caller.get_text()
        self._config["shuffle"] = self.shuffle.get_active()
        self._config["is-browser"] = False
        
        playlist = self._library.get_tracks(self._config)
        
        self._set_playlist(playlist)
        
        self._config.notify("search-str")

    def _set_playlist(self, data):
        self._config["pos"] = 0

        self._player.playlist.update(data)
        
        if self._config["shuffle"]:
            self._player.playlist.shuffle(True)
    
    def browse_mode(self):
        self._search_label.hide()
        self._search_entry.hide()
        self._browse_button.show()
        
        if self._config["is-browser"]:
            self._artist.show()
            self._album.show()
        else:
            self._show_artists()
    
    def search_mode(self):
        self._browse_button.hide()
        self._artist.hide()
        self._album.hide()
        self._search_label.show()
        self._search_entry.show()
        
        if self._browser.get_parent() is not None:
            self._show_playlist()
    
    def _show_playlist(self):
        if self._playlist.get_parent() is None:
            self._scroll.remove(self._browser)
            self._scroll.add(self._playlist)

    def _show_browser(self):
        if self._browser.get_parent() is None:
            self._scroll.remove(self._playlist)
            self._scroll.add(self._browser)

    def _on_search_artist(self, client, cxn_id, entry, data):
        val = entry.get_value().get_string()

        val = val if val != "" else _("All Artists")
        
        self._artist.set_text(val)
        self._artist.show()

        self._album.hide()
        print "search-artist changed"

    def _on_search_album(self, client, cxn_id, entry, data):
        val = entry.get_value().get_string()
  
        val = val if val != "" else _("All Albums")
        
        self._album.set_text(val)
        self._album.show()
        
        self._browse_complete()
        print "search-album changed"

    def _select_album(self, caller):        
        if not (self._browser._state & ALBUM_SELECTED):
            self._browser._state |= ALBUM_SELECTED
            self._config.notify("search-album")
        
    def _show_albums(self, caller):
        if not (self._browser._state & ALBUM_SELECTED):
            self._config.notify("search-artist")

        self._show_browser()
        self._browser.show_albums()
        
        self._browser._state = ARTIST_SELECTED

    def _show_artists(self, caller=None):
        self._show_browser()
        self._browser.show_artists()
        self._browser._state = NOTHING_SELECTED

class PlaylistView(gtk.TreeView):
    def __init__(self, model, config, player):        
        gtk.TreeView.__init__(self, model)
        
        self._config = config
        self._player = player
        
        self.set_rules_hint(True)
        self.set_headers_visible(True)
        self.set_fixed_height_mode(True)
        keys = ("title", "artist", "album")
        cols = (_("Title"), _("Artist"), _("Album"))
        
        cell = gtk.CellRendererText()
        cell.set_property("ellipsize", pango.ELLIPSIZE_END)
            
        for i in xrange(len(keys)):
            col = gtk.TreeViewColumn(cols[i], cell)
            col.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
            col.set_fixed_width(200)
            col.set_cell_data_func(cell, self._data_func, keys[i])
            self.append_column(col)
        
        # drag and drop
        if not IS_MAEMO:
            self.drag_dest_set(gtk.DEST_DEFAULT_DROP, [("text/uri-list", 0, 0)], gtk.gdk.ACTION_LINK)
            self.connect("drag-data-received", self._recieve_drag_data)
            self.enable_model_drag_source(gtk.gdk.BUTTON1_MASK, [("text/uri-list", 0, 0)], gtk.gdk.ACTION_COPY)
            self.connect("drag-data-get", self._get_drag_data)
           
        # reset config 
        self._on_pos_changed()
        
        # Signals
        self._config.notify_add("pos", self._on_pos_changed)
        self.connect("row-activated", self._on_row_activated)
    
    def _recieve_drag_data(self, tv, ctxt, x, y, selection, *args):
        print selection.get_uris()
    
    def _get_drag_data(self, tv, ctxt, selection, *args):
        uri = self._player.playlist[self._config["pos"]].uri
        selection.set_uris([uri])

    def _on_pos_changed(self, *args):
        self.set_cursor(self._config["pos"])
 
    def _on_row_activated(self, caller, path, column):
        self._player.goto_pos(path[0])

    def _data_func(self, col, cell, model, itr, key):
        cell.set_property("text", model[itr][key])
